From 06250a10f7aa775cd8e08e65c18eeb71d27086a6 Mon Sep 17 00:00:00 2001
From: Torsten Polle <Torsten.Polle@gmx.de>
Date: Mon, 19 Mar 2018 21:37:25 +0100
Subject: [PATCH] runtime/unwind.c: Support KASLR.

---
 runtime/unwind.c | 5 ++++-
 1 file changed, 4 insertions(+), 1 deletion(-)

diff --git a/runtime/unwind.c b/runtime/unwind.c
index ec7cd58..5fc4ad4 100644
--- a/runtime/unwind.c
+++ b/runtime/unwind.c
@@ -724,10 +724,13 @@ adjustStartLoc (unsigned long startLoc,
   dbug_unwind(2, "adjustStartLoc=%lx, ptrType=%s, m=%s, s=%s eh=%d\n",
 	      startLoc, _stp_eh_enc_name(ptrType), m->path, s->name, is_ehframe);
   if (startLoc == 0
-      || strcmp (m->name, "kernel")  == 0
       || (strcmp (s->name, ".absolute") == 0 && !is_ehframe))
     return startLoc;
 
+  /* Even the kernel might have been relocated due to KASLR. */
+  if (strcmp (m->name, "kernel")  == 0)
+    return startLoc + (s->static_addr & ~0x1fffff) - s->sec_load_offset;
+
   /* eh_frame data has been loaded in the kernel, so readjust offset. */
   if (is_ehframe) {
     dbug_unwind(2, "eh_frame=%lx, eh_frame_addr=%lx\n", (unsigned long) m->eh_frame, m->eh_frame_addr);
-- 
2.7.4

